/**
 * 站内信功能模块模型
 * @author Eagle <eagle@jingkaigz.com>
 */
define(function(require) {
	var r = require("../request");
	var app = require("../app");
	var userInfo = app.getUser() || {};
	const CATE = 1; // 1:孕妇
	
	/**
	 * 根据登录用户获取所有的类型消息和最后一条消息
	 * @param function callback 回调函数
	 * @author Eagle <eagle@jingkaigz.com>
	 */
	var getAllByLoginUser = function(callback) {
		var params = {
			usid : userInfo.id,
			cate : CATE
		}
		
		r.ajax({
			url : "communication/Communication/calMailAll",
			method : "POST",
			data : params,
			token : true,
			onSuccess : function(data) {
				callback(true, data.msg, data);
			},
			onFail : function(data) {
				callback(false, data.msg, data);
			},
			onError : function(data) {
				callback(false, r.requestErrorMsg);
			}
		})
	};
	
	/**
	 * 根据登录用户获取指定类型的消息列表
	 * @param string type 消息类型
	 * @param int pageNumber 页码
	 * @param int pageSize 每页数量， 0表示全部取回
	 * @param function callback 回调函数
	 * @author Eagle <eagle@jingkaigz.com>
	 */
	var getListByLoginUser = function(type, pageNumber, pageSize, callback) {
		var params = {
				pageNum : pageNumber || 1,
				pageSize : pageSize || 0,
				filters: {
					usid : userInfo.id,
					cate : CATE,
					type : type
				},
			}
			
		r.ajax({
			url : "communication/Communication/listMail",
			method : "POST",
			data : params,
			token : true,
			onSuccess : function(data) {
				callback(true, data.msg, data.content);
			},
			onFail : function(data) {
				callback(false, data.msg, "");
			},
			onError : function(data) {
				callback(false, r.requestErrorMsg);
			}
		})
		
	};
	
	/**
	 * 根据类型设置消息全部为已读状态
	 * @param string type 消息类型
	 * @param function callback 回调函数
	 * @author Eagle <eagle@jingkaigz.com>
	 */
	var setReadByType = function(type, callback) {
		var params = {
			usid : userInfo.id,
			cate : 1,
			type : type
		}
		
		r.ajax({
			url : "communication/Communication/updateMail",
			method : "POST",
			data : params,
			token : true,
			onSuccess : function(data) {
				callback(true, data.msg, data.content);
			},
			onFail : function(data) {
				callback(false, data.msg, "");
			},
			onError : function(data) {
				callback(false, r.requestErrorMsg);
			}
		})
	};
	
	/**
	 * 搜索用户
	 * @param string account 搜索账号
	 * @param int pageNumber 页码
	 * @param int pageSize 每页数量， 0表示全部取回
	 * @param function callback 回调函数
	 * @author Eagle <eagle@jingkaigz.com>
	 */
	var getListByAcount = function(account, pageNumber, pageSize, callback) {
		var params = {
				pageNum : pageNumber || 1,
				pageSize : pageSize || 0,
				filters: {
					account : account
				},
			}
		
		if(account.length <= 0) {
			return callback(false, "请输入搜索内容");
		}
			
		r.ajax({
			url : "communication/Communication/listUserSearch",
			method : "POST",
			data : params,
			token : true,
			onSuccess : function(data) {
				callback(true, data.msg, data.content);
			},
			onFail : function(data) {
				callback(false, data.msg, "");
			},
			onError : function(data) {
				callback(false, r.requestErrorMsg);
			}
		})
		
	};
	
	return {
		getAllByLoginUser: getAllByLoginUser,
		getListByLoginUser : getListByLoginUser,
		setReadByType : setReadByType,
		getListByAcount: getListByAcount,
	};
});